# -*- coding: utf-8 -*-
from App.config import getConfiguration
from collective.fingerpointing.config import AUDITLOG
from collective.fingerpointing.config import PROJECTNAME

import logging
import os.path
import time
import zc.lockfile


FORMAT = '%(asctime)s - %(levelname)s - %(message)s'

# by default, the audit log will use the same location used for the event log
eventlog = getattr(getConfiguration(), 'eventlog', None)

if eventlog is None:
    # we are running tests
    logfile = os.path.join('.', AUDITLOG)
else:
    try:
        logpath = eventlog.handler_factories[0].instance.baseFilename
        logfolder = os.path.split(logpath)[0]
        logfile = os.path.join(logfolder, AUDITLOG)
    except AttributeError:
        # we are in the debug console
        logfile = os.path.join('.', AUDITLOG)

logger = logging.getLogger(PROJECTNAME)
logger.setLevel(logging.INFO)
logger.info('Start logging audit information to ' + AUDITLOG)

# support automatic rotation of audit log files at timed intervals
# we can later implement a way to make this configurable
handler = logging.handlers.TimedRotatingFileHandler(
    logfile, when='midnight', backupCount=30)

formatter = logging.Formatter(FORMAT)
handler.setFormatter(formatter)
logger.addHandler(handler)


def log_info(*args, **kwargs):
    """Log information to a file handling access from multiple instances.
    This code was taken from ZEO/ClientStorage.py.
    """
    # try to lock the logfile then write to the logfile
    lockfilename = logfile + '.lock'
    n = 0

    while 1:
        try:
            lock = zc.lockfile.LockFile(lockfilename)
            logger.info(*args, **kwargs)
            lock.close()
        except zc.lockfile.LockError:
            time.sleep(0.01)
            n += 1
            if n > 60000:
                raise
        else:
            break

    return



# -*- coding: utf-8 -*-
from zope.globalrequest import getRequest


def get_request_information():
    try:
        request = getRequest()
        user = request['AUTHENTICATED_USER']
        ip = request['REMOTE_ADDR']
        return user, ip
    except (KeyError, TypeError):
        return 'test', '127.0.0.1'  # running tests



# -*- coding: utf-8 -*-
from .iterate_logger import iterate_logger  # noqa
from .lifecycle_logger import lifecycle_logger  # noqa
from .pas_logger import pas_logger  # noqa
from .registry_logger import registry_logger  # noqa



# -*- coding: utf-8 -*-
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from plone import api
from plone.api.exc import InvalidParameterError
from Products.PluggableAuthService.interfaces.events import IPrincipalCreatedEvent
from Products.PluggableAuthService.interfaces.events import IPrincipalDeletedEvent
from Products.PluggableAuthService.interfaces.events import IUserLoggedInEvent
from Products.PluggableAuthService.interfaces.events import IUserLoggedOutEvent


def pas_logger(event):
    """Log authentication events like users logging in and loggin out."""
    # subscriber is registered even if package has not yet been installed
    # ignore any error caused by missing registry records
    try:
        record = IFingerPointingSettings.__identifier__ + '.audit_pas'
        audit_pas = api.portal.get_registry_record(record)
    except InvalidParameterError:
        return

    if audit_pas:
        user, ip = get_request_information()

        if IUserLoggedInEvent.providedBy(event):
            action = 'login'
            extras = ''
        elif IUserLoggedOutEvent.providedBy(event):
            action = 'logout'
            extras = ''
        elif IPrincipalCreatedEvent.providedBy(event):
            action = 'create'
            extras = u'principal={0}'.format(event.principal)
        elif IPrincipalDeletedEvent.providedBy(event):
            action = 'remove'
            extras = u'principal={0}'.format(event.principal)

        log_info(AUDIT_MESSAGE.format(user, ip, action, extras))



# -*- coding: utf-8 -*-
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.interfaces import IFingerPointingSettings
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from plone import api
from plone.api.exc import InvalidParameterError
from plone.registry.interfaces import IRecordModifiedEvent


def registry_logger(event):
    """Log registry events like records being modified."""
    # subscriber is registered even if package has not yet been installed
    # ignore any error caused by missing registry records
    try:
        record = IFingerPointingSettings.__identifier__ + '.audit_registry'
        audit_registry = api.portal.get_registry_record(record)
    except InvalidParameterError:
        return

    if audit_registry:
        user, ip = get_request_information()

        if IRecordModifiedEvent.providedBy(event):
            action = 'modify'
            extras = u'object={0} value={1}'.format(event.record, event.record.value)

        log_info(AUDIT_MESSAGE.format(user, ip, action, extras))
